@@ -28,13 +28,11 @@ from threading import RLock
 import json
 from functools import reduce
 
-from pyasn1.codec.ber import encoder
-
-from .. import ANONYMOUS, SIMPLE, SASL, MODIFY_ADD, MODIFY_DELETE, MODIFY_REPLACE, \
+from .. import ANONYMOUS, SIMPLE, SASL, MODIFY_ADD, MODIFY_DELETE, MODIFY_REPLACE, get_config_parameter, \
     DEREF_ALWAYS, SUBTREE, ASYNC, SYNC, CLIENT_STRATEGIES, RESULT_SUCCESS, RESULT_COMPARE_TRUE, NO_ATTRIBUTES, ALL_ATTRIBUTES, \
     ALL_OPERATIONAL_ATTRIBUTES, MODIFY_INCREMENT, LDIF, SASL_AVAILABLE_MECHANISMS, \
-    RESTARTABLE, ROUND_ROBIN, REUSABLE, DEFAULT_THREADED_POOL_NAME, AUTO_BIND_NONE, AUTO_BIND_TLS_BEFORE_BIND, AUTO_BIND_TLS_AFTER_BIND, \
-    AUTO_BIND_NO_TLS, STRING_TYPES, SEQUENCE_TYPES, MOCK_SYNC, MOCK_ASYNC, NTLM
+    RESTARTABLE, ROUND_ROBIN, REUSABLE, AUTO_BIND_NONE, AUTO_BIND_TLS_BEFORE_BIND, AUTO_BIND_TLS_AFTER_BIND, \
+    AUTO_BIND_NO_TLS, STRING_TYPES, SEQUENCE_TYPES, MOCK_SYNC, MOCK_ASYNC, NTLM, EXTERNAL, DIGEST_MD5, GSSAPI
 from ..extend import ExtendedOperationsRoot
 from .pooling import ServerPool
 from .server import Server
@@ -56,7 +54,7 @@ from ..strategy.ldifProducer import LdifProducerStrategy
 from ..strategy.sync import SyncStrategy
 from ..strategy.restartable import RestartableStrategy
 from ..operation.unbind import unbind_operation
-from ..protocol.rfc2696 import RealSearchControlValue, Cookie, Size
+from ..protocol.rfc2696 import paged_search_control
 from .usage import ConnectionUsage
 from .tls import Tls
 from .exceptions import LDAPUnknownStrategyError, LDAPBindError, LDAPUnknownAuthenticationMethodError, \
@@ -79,6 +77,11 @@ def _format_socket_endpoint(endpoint):
     elif endpoint and len(endpoint) == 4:  # IPv6
         return '[' + str(endpoint[0]) + ']:' + str(endpoint[1])
 
+    try:
+        return str(endpoint)
+    except Exception:
+        return '?'
+
 
 def _format_socket_endpoints(sock):
     if sock:
@@ -167,7 +170,8 @@ class Connection(object):
                  raise_exceptions=False,
                  pool_name=None,
                  pool_size=None,
-                 pool_lifetime=None):
+                 pool_lifetime=None,
+                 fast_decoder=True):
 
         self.lock = RLock()  # re-entrant lock to assure that operations in the Connection object are executed atomically in the same thread
         with self.lock:
@@ -220,7 +224,7 @@ class Connection(object):
             self._bind_controls = None
             self._executing_deferred = False
             self.lazy = lazy
-            self.pool_name = pool_name if pool_name else DEFAULT_THREADED_POOL_NAME
+            self.pool_name = pool_name if pool_name else get_config_parameter('DEFAULT_THREADED_POOL_NAME')
             self.pool_size = pool_size
             self.pool_lifetime = pool_lifetime
             self.starting_tls = False
@@ -228,7 +232,8 @@ class Connection(object):
             self.raise_exceptions = raise_exceptions
             self.auto_range = True if auto_range else False
             self.extend = ExtendedOperationsRoot(self)
-            self._entries = None
+            self._entries = []
+            self.fast_decoder = fast_decoder
 
             if isinstance(server, STRING_TYPES):
                 server = Server(server)
@@ -304,7 +309,8 @@ class Connection(object):
             _format_socket_endpoints(self.socket),
             'tls not started' if not self.tls_started else('deferred start_tls' if self._deferred_start_tls else 'tls started'),
             'listening' if self.listening else 'not listening',
-            self.strategy.__class__.__name__ if hasattr(self, 'strategy') else 'No strategy'
+            self.strategy.__class__.__name__ if hasattr(self, 'strategy') else 'No strategy',
+            'internal decoder' if self.fast_decoder else 'pyasn1 decoder'
         ]
         return ' - '.join(s)
 
@@ -327,9 +333,10 @@ class Connection(object):
         r += '' if self.read_only is None else ', read_only={0.read_only!r}'.format(self)
         r += '' if self.lazy is None else ', lazy={0.lazy!r}'.format(self)
         r += '' if self.raise_exceptions is None else ', raise_exceptions={0.raise_exceptions!r}'.format(self)
-        r += '' if (self.pool_name is None or self.pool_name == DEFAULT_THREADED_POOL_NAME) else ', pool_name={0.pool_name!r}'.format(self)
+        r += '' if (self.pool_name is None or self.pool_name == get_config_parameter('DEFAULT_THREADED_POOL_NAME')) else ', pool_name={0.pool_name!r}'.format(self)
         r += '' if self.pool_size is None else ', pool_size={0.pool_size!r}'.format(self)
         r += '' if self.pool_lifetime is None else ', pool_lifetime={0.pool_lifetime!r}'.format(self)
+        r += '' if self.fast_decoder is None else (', fast_decoder=' + 'True' if self.fast_decoder else 'False')
         r += ')'
 
         return r
@@ -347,7 +354,7 @@ class Connection(object):
         r += '' if self.strategy_type is None else ', client_strategy={0.strategy_type!r}'.format(self)
         r += '' if self.auto_referrals is None else ', auto_referrals={0.auto_referrals!r}'.format(self)
         r += '' if self.sasl_mechanism is None else ', sasl_mechanism={0.sasl_mechanism!r}'.format(self)
-        if self.sasl_mechanism == 'DIGEST-MD5':
+        if self.sasl_mechanism == DIGEST_MD5:
             r += '' if self.sasl_credentials is None else ", sasl_credentials=({0!r}, {1!r}, '{2}', {3!r})".format(self.sasl_credentials[0], self.sasl_credentials[1], '*' * len(self.sasl_credentials[2]), self.sasl_credentials[3])
         else:
             r += '' if self.sasl_credentials is None else ', sasl_credentials={0.sasl_credentials!r}'.format(self)
@@ -357,7 +364,7 @@ class Connection(object):
         r += '' if self.lazy is None else ', lazy={0.lazy!r}'.format(self)
         r += '' if self.raise_exceptions is None else ', raise_exceptions={0.raise_exceptions!r}'.format(self)
         r += '' if (
-            self.pool_name is None or self.pool_name == DEFAULT_THREADED_POOL_NAME) else ', pool_name={0.pool_name!r}'.format(
+            self.pool_name is None or self.pool_name == get_config_parameter('DEFAULT_THREADED_POOL_NAME')) else ', pool_name={0.pool_name!r}'.format(
             self)
         r += '' if self.pool_size is None else ', pool_size={0.pool_size!r}'.format(self)
         r += '' if self.pool_lifetime is None else ', pool_lifetime={0.pool_lifetime!r}'.format(self)
@@ -512,7 +519,7 @@ class Connection(object):
 
                 if read_server_info and self.bound:
                     self.refresh_server_info()
-            self._entries = None
+            self._entries = []
 
             if log_enabled(BASIC):
                 log(BASIC, 'done BIND operation, result <%s>', self.bound)
@@ -583,7 +590,7 @@ class Connection(object):
             if not attributes:
                 attributes = [NO_ATTRIBUTES]
             elif attributes == ALL_ATTRIBUTES:
-                attributes = ['*']
+                attributes = [ALL_ATTRIBUTES]
 
             if get_operational_attributes and isinstance(attributes, list):
                 attributes.append(ALL_OPERATIONAL_ATTRIBUTES)
@@ -593,18 +600,19 @@ class Connection(object):
             if isinstance(paged_size, int):
                 if log_enabled(PROTOCOL):
                     log(PROTOCOL, 'performing paged search for %d items with cookie <%s> for <%s>', paged_size, escape_bytes(paged_cookie), self)
-                real_search_control_value = RealSearchControlValue()
-                real_search_control_value['size'] = Size(paged_size)
-                real_search_control_value['cookie'] = Cookie(paged_cookie) if paged_cookie else Cookie('')
+                # real_search_control_value = RealSearchControlValue()
+                # real_search_control_value['size'] = Size(paged_size)
+                # real_search_control_value['cookie'] = Cookie(paged_cookie) if paged_cookie else Cookie('')
                 if controls is None:
                     controls = []
-                controls.append(('1.2.840.113556.1.4.319', paged_criticality if isinstance(paged_criticality, bool) else False, encoder.encode(real_search_control_value)))
+                # controls.append(('1.2.840.113556.1.4.319', paged_criticality if isinstance(paged_criticality, bool) else False, encoder.encode(real_search_control_value)))
+                controls.append(paged_search_control(paged_criticality, paged_size, paged_cookie))
 
             request = search_operation(search_base, search_filter, search_scope, dereference_aliases, attributes, size_limit, time_limit, types_only, self.server.schema if self.server else None)
             if log_enabled(PROTOCOL):
                 log(PROTOCOL, 'SEARCH request <%s> sent via <%s>', search_request_to_dict(request), self)
             response = self.post_send_search(self.send('searchRequest', request, controls))
-            self._entries = None
+            self._entries = []
 
             if isinstance(response, int):
                 return_value = response
@@ -641,7 +649,7 @@ class Connection(object):
             if log_enabled(PROTOCOL):
                 log(PROTOCOL, 'COMPARE request <%s> sent via <%s>', compare_request_to_dict(request), self)
             response = self.post_send_single_response(self.send('compareRequest', request, controls))
-            self._entries = None
+            self._entries = []
             if isinstance(response, int):
                 return_value = response
                 if log_enabled(PROTOCOL):
@@ -703,7 +711,7 @@ class Connection(object):
             if log_enabled(PROTOCOL):
                 log(PROTOCOL, 'ADD request <%s> sent via <%s>', add_request_to_dict(request), self)
             response = self.post_send_single_response(self.send('addRequest', request, controls))
-            self._entries = None
+            self._entries = []
 
             if isinstance(response, STRING_TYPES + (int, )):
                 return_value = response
@@ -740,7 +748,7 @@ class Connection(object):
             if log_enabled(PROTOCOL):
                 log(PROTOCOL, 'DELETE request <%s> sent via <%s>', delete_request_to_dict(request), self)
             response = self.post_send_single_response(self.send('delRequest', request, controls))
-            self._entries = None
+            self._entries = []
 
             if isinstance(response, STRING_TYPES + (int, )):
                 return_value = response
@@ -813,7 +821,7 @@ class Connection(object):
             if log_enabled(PROTOCOL):
                 log(PROTOCOL, 'MODIFY request <%s> sent via <%s>', modify_request_to_dict(request), self)
             response = self.post_send_single_response(self.send('modifyRequest', request, controls))
-            self._entries = None
+            self._entries = []
 
             if isinstance(response, STRING_TYPES + (int, )):
                 return_value = response
@@ -860,7 +868,7 @@ class Connection(object):
             if log_enabled(PROTOCOL):
                 log(PROTOCOL, 'MODIFY DN request <%s> sent via <%s>', modify_dn_request_to_dict(request), self)
             response = self.post_send_single_response(self.send('modDNRequest', request, controls))
-            self._entries = None
+            self._entries = []
 
             if isinstance(response, STRING_TYPES + (int, )):
                 return_value = response
@@ -896,7 +904,7 @@ class Connection(object):
                     self.send('abandonRequest', request, controls)
                     self.result = None
                     self.response = None
-                    self._entries = None
+                    self._entries = []
                     return_value = True
                 else:
                     if log_enabled(ERROR):
@@ -923,7 +931,7 @@ class Connection(object):
             if log_enabled(PROTOCOL):
                 log(PROTOCOL, 'EXTENDED request <%s> sent via <%s>', extended_request_to_dict(request), self)
             response = self.post_send_single_response(self.send('extendedReq', request, controls))
-            self._entries = None
+            self._entries = []
             if isinstance(response, int):
                 return_value = response
                 if log_enabled(PROTOCOL):
@@ -977,11 +985,11 @@ class Connection(object):
             result = None
             if not self.sasl_in_progress:
                 self.sasl_in_progress = True
-                if self.sasl_mechanism == 'EXTERNAL':
+                if self.sasl_mechanism == EXTERNAL:
                     result = sasl_external(self, controls)
-                elif self.sasl_mechanism == 'DIGEST-MD5':
+                elif self.sasl_mechanism == DIGEST_MD5:
                     result = sasl_digest_md5(self, controls)
-                elif self.sasl_mechanism == 'GSSAPI':
+                elif self.sasl_mechanism == GSSAPI:
                     from ..protocol.sasl.kerberos import sasl_gssapi  # needs the gssapi package
                     result = sasl_gssapi(self, controls)
 
