@@ -355,8 +355,12 @@ class LDAPPackageUnavailableError(LDAPConfigurationError, ImportError):
     pass
 
 
+class LDAPConfigurationParameterError(LDAPConfigurationError):
+    pass
+
+
 # abstract layer exceptions
-class LDAPKeyError(LDAPExceptionError, KeyError):
+class LDAPKeyError(LDAPExceptionError, KeyError, AttributeError):
     pass
 
 
@@ -422,7 +426,7 @@ class LDAPServerPoolExhaustedError(LDAPExceptionError):
     pass
 
 
-class LDAPInvalidPort(LDAPExceptionError):
+class LDAPInvalidPortError(LDAPExceptionError):
     pass
 
 
@@ -511,7 +515,7 @@ class LDAPSocketSendError(LDAPCommunicationError, socket.error):
     pass
 
 
-class LDAPSessionTerminatedByServer(LDAPCommunicationError):
+class LDAPSessionTerminatedByServerError(LDAPCommunicationError):
     pass
 
 
