@@ -29,11 +29,8 @@ from time import sleep
 from random import choice
 from datetime import datetime
 
-from pyasn1.codec.ber import encoder, decoder
-
-from .. import SESSION_TERMINATED_BY_SERVER, RESPONSE_SLEEPTIME, RESPONSE_WAITING_TIMEOUT, SYNC, ANONYMOUS,\
-    DO_NOT_RAISE_EXCEPTIONS, RESULT_REFERRAL, RESPONSE_COMPLETE, BASE
-from ..core.exceptions import LDAPOperationResult, LDAPSASLBindInProgressError, LDAPSocketOpenError, LDAPSessionTerminatedByServer,\
+from .. import SESSION_TERMINATED_BY_SERVER, SYNC, ANONYMOUS, get_config_parameter, DO_NOT_RAISE_EXCEPTIONS, RESULT_REFERRAL, RESPONSE_COMPLETE, BASE
+from ..core.exceptions import LDAPOperationResult, LDAPSASLBindInProgressError, LDAPSocketOpenError, LDAPSessionTerminatedByServerError,\
     LDAPUnknownResponseError, LDAPUnknownRequestError, LDAPReferralError, communication_exception_factory, \
     LDAPSocketSendError, LDAPExceptionError, LDAPControlsError, LDAPResponseTimeoutError
 from ..utils.uri import parse_uri
@@ -41,10 +38,11 @@ from ..protocol.rfc4511 import LDAPMessage, ProtocolOp, MessageID
 from ..operation.add import add_response_to_dict, add_request_to_dict
 from ..operation.modify import modify_request_to_dict, modify_response_to_dict
 from ..operation.search import search_result_reference_response_to_dict, search_result_done_response_to_dict,\
-    search_result_entry_response_to_dict, search_request_to_dict
-from ..operation.bind import bind_response_to_dict, bind_request_to_dict
+    search_result_entry_response_to_dict, search_request_to_dict, search_result_entry_response_to_dict_fast, search_result_reference_response_to_dict_fast
+from ..operation.bind import bind_response_to_dict, bind_request_to_dict, sicily_bind_response_to_dict, bind_response_to_dict_fast, \
+    sicily_bind_response_to_dict_fast
 from ..operation.compare import compare_response_to_dict, compare_request_to_dict
-from ..operation.extended import extended_request_to_dict, extended_response_to_dict, intermediate_response_to_dict
+from ..operation.extended import extended_request_to_dict, extended_response_to_dict, intermediate_response_to_dict, extended_response_to_dict_fast, intermediate_response_to_dict_fast
 from ..core.server import Server
 from ..operation.modifyDn import modify_dn_request_to_dict, modify_dn_response_to_dict
 from ..operation.delete import delete_response_to_dict, delete_request_to_dict
@@ -53,6 +51,9 @@ from ..operation.abandon import abandon_request_to_dict
 from ..core.tls import Tls
 from ..protocol.oid import Oids
 from ..protocol.rfc2696 import RealSearchControlValue
+from ..protocol.microsoft import DirSyncControlResponseValue
+from ..utils.log import log, log_enabled, ERROR, BASIC, PROTOCOL, NETWORK, EXTENDED, format_ldap_message
+from ..utils.asn1 import encoder, decoder, ldap_result_to_dict_fast, decode_sequence
 
 
 # noinspection PyProtectedMember
@@ -69,14 +70,31 @@ class BaseStrategy(object):
         self.no_real_dsa = None  # indicates a connection to a fake LDAP server
         self.pooled = None  # Indicates a connection with a connection pool
         self.can_stream = None  # indicate if a strategy keeps a stream of responses (i.e. LdifProducer can accumulate responses with a single header). Stream must be initialized and closed in _start_listen() and _stop_listen()
+        if log_enabled(BASIC):
+            log(BASIC, 'instantiated <%s>: <%s>', self.__class__.__name__, self)
+
+    def __str__(self):
+        s = [
+            str(self.connection) if self.connection else 'None',
+            'sync' if self.sync else 'async',
+            'no real DSA' if self.no_real_dsa else 'real DSA',
+            'pooled' if self.pooled else 'not pooled',
+            'can stream output' if self.pooled else 'cannot stream output',
+        ]
+        return ' - '.join(s)
 
     def open(self, reset_usage=True, read_server_info=True):
         """
         Open a socket to a server. Choose a server from the server pool if available
         """
+        if log_enabled(NETWORK):
+            log(NETWORK, 'opening connection for <%s>', self.connection)
         if self.connection.lazy and not self.connection._executing_deferred:
             self.connection._deferred_open = True
             self.connection.closed = False
+            if log_enabled(NETWORK):
+                log(NETWORK, 'deferring open connection for <%s>', self.connection)
+
         else:
             if not self.connection.closed and not self.connection._executing_deferred:  # try to close connection if still open
                 self.close()
@@ -97,7 +115,9 @@ class BaseStrategy(object):
             if not self.no_real_dsa:
                 for candidate_address in self.connection.server.candidate_addresses():
                     try:
-                        self._open_socket(candidate_address, self.connection.server.ssl)
+                        if log_enabled(BASIC):
+                            log(BASIC, 'try to open candidate address %s', candidate_address[:-2])
+                        self._open_socket(candidate_address, self.connection.server.ssl, unix_socket=self.connection.server.ipc)
                         self.connection.server.current_address = candidate_address
                         self.connection.server.update_availability(candidate_address, True)
                         break
@@ -106,25 +126,42 @@ class BaseStrategy(object):
                         exception_history.append((datetime.now(), exc_info()[0], exc_info()[1], candidate_address[4]))
 
                 if not self.connection.server.current_address and exception_history:
-                    raise LDAPSocketOpenError('unable to open socket', exception_history)
+                    if len(exception_history) == 1:  # only one exception, reraise
+                        if log_enabled(ERROR):
+                            log(ERROR, '<%s> for <%s>', exception_history[0][1](exception_history[0][2]), self.connection)
+                        raise exception_history[0][1](exception_history[0][2])
+                    else:
+                        if log_enabled(ERROR):
+                            log(ERROR, 'unable to open socket for <%s>', self.connection)
+                        raise LDAPSocketOpenError('unable to open socket', exception_history)
                 elif not self.connection.server.current_address:
+                    if log_enabled(ERROR):
+                        log(ERROR, 'invalid server address for <%s>', self.connection)
                     raise LDAPSocketOpenError('invalid server address')
 
             self.connection._deferred_open = False
             self._start_listen()
+            if log_enabled(NETWORK):
+                log(NETWORK, 'connection open for <%s>', self.connection)
 
     def close(self):
         """
         Close connection
         """
+        if log_enabled(NETWORK):
+            log(NETWORK, 'closing connection for <%s>', self.connection)
         if self.connection.lazy and not self.connection._executing_deferred and (self.connection._deferred_bind or self.connection._deferred_open):
             self.connection.listening = False
             self.connection.closed = True
+            if log_enabled(NETWORK):
+                log(NETWORK, 'deferred connection closed for <%s>', self.connection)
         else:
             if not self.connection.closed:
                 self._stop_listen()
                 if not self. no_real_dsa:
                     self._close_socket()
+            if log_enabled(NETWORK):
+                log(NETWORK, 'connection closed for <%s>', self.connection)
 
         self.connection.bound = False
         self.connection.request = None
@@ -138,7 +175,7 @@ class BaseStrategy(object):
         if self.connection.usage:
             self.connection._usage.stop()
 
-    def _open_socket(self, address, use_ssl=False):
+    def _open_socket(self, address, use_ssl=False, unix_socket=False):
         """
         Tries to open and connect a socket to a Server
         raise LDAPExceptionError if unable to open or connect socket
@@ -151,12 +188,15 @@ class BaseStrategy(object):
             exc = e
 
         if exc:
+            if log_enabled(ERROR):
+                log(ERROR, '<%s> for <%s>', self.connection.last_error, self.connection)
+
             raise communication_exception_factory(LDAPSocketOpenError, exc)(self.connection.last_error)
 
         try:
             if self.connection.server.connect_timeout:
                 self.connection.socket.settimeout(self.connection.server.connect_timeout)
-            self.connection.socket.connect(self.connection.server.address_info[0][4])
+            self.connection.socket.connect(address[4])
             if self.connection.server.connect_timeout:
                 self.connection.socket.settimeout(None)  # disable socket timeout - socket is in blocking mode
         except socket.error as e:
@@ -164,6 +204,8 @@ class BaseStrategy(object):
             exc = e
 
         if exc:
+            if log_enabled(ERROR):
+                log(ERROR, '<%s> for <%s>', self.connection.last_error, self.connection)
             raise communication_exception_factory(LDAPSocketOpenError, exc)(self.connection.last_error)
 
         if use_ssl:
@@ -176,6 +218,8 @@ class BaseStrategy(object):
                 exc = e
 
             if exc:
+                if log_enabled(ERROR):
+                    log(ERROR, '<%s> for <%s>', self.connection.last_error, self.connection)
                 raise communication_exception_factory(LDAPSocketOpenError, exc)(self.connection.last_error)
 
         if self.connection.usage:
@@ -213,11 +257,12 @@ class BaseStrategy(object):
         Send an LDAP message
         Returns the message_id
         """
-        exc = None
         self.connection.request = None
         if self.connection.listening:
             if self.connection.sasl_in_progress and message_type not in ['bindRequest']:  # as per RFC4511 (4.2.1)
                 self.connection.last_error = 'cannot send operation requests while SASL bind is in progress'
+                if log_enabled(ERROR):
+                    log(ERROR, '<%s> for <%s>', self.connection.last_error, self.connection)
                 raise LDAPSASLBindInProgressError(self.connection.last_error)
             message_id = self.connection.server.next_message_id()
             ldap_message = LDAPMessage()
@@ -226,30 +271,19 @@ class BaseStrategy(object):
             message_controls = build_controls_list(controls)
             if message_controls is not None:
                 ldap_message['controls'] = message_controls
-
-            try:
-                encoded_message = encoder.encode(ldap_message)
-                self.connection.socket.sendall(encoded_message)
-            except socket.error as e:
-                self.connection.last_error = 'socket sending error' + str(e)
-                encoded_message = None
-                exc = e
-
-            if exc:
-                raise communication_exception_factory(LDAPSocketSendError, exc)(self.connection.last_error)
-
             self.connection.request = BaseStrategy.decode_request(ldap_message)
             self.connection.request['controls'] = controls
             self._outstanding[message_id] = self.connection.request
-            if self.connection.usage:
-                self.connection._usage.update_transmitted_message(self.connection.request, len(encoded_message))
+            self.sending(ldap_message)
         else:
             self.connection.last_error = 'unable to send message, socket is not open'
+            if log_enabled(ERROR):
+                log(ERROR, '<%s> for <%s>', self.connection.last_error, self.connection)
             raise LDAPSocketOpenError(self.connection.last_error)
 
         return message_id
 
-    def get_response(self, message_id, timeout=RESPONSE_WAITING_TIMEOUT):
+    def get_response(self, message_id, timeout=None):
         """
         Get response LDAP messages
         Responses are returned by the underlying connection strategy
@@ -258,14 +292,16 @@ class BaseStrategy(object):
         Responses without result is stored in connection.response
         A tuple (responses, result) is returned
         """
+        if timeout is None:
+            timeout = get_config_parameter('RESPONSE_WAITING_TIMEOUT')
         response = None
         result = None
         if self._outstanding and message_id in self._outstanding:
             while timeout >= 0:  # waiting for completed message to appear in responses
                 responses = self._get_response(message_id)
                 if not responses:
-                    sleep(RESPONSE_SLEEPTIME)
-                    timeout -= RESPONSE_SLEEPTIME
+                    sleep(get_config_parameter('RESPONSE_SLEEPTIME'))
+                    timeout -= get_config_parameter('RESPONSE_SLEEPTIME')
                     continue
 
                 if responses == SESSION_TERMINATED_BY_SERVER:
@@ -274,7 +310,9 @@ class BaseStrategy(object):
                     except (socket.error, LDAPExceptionError):
                         pass
                     self.connection.last_error = 'session terminated by server'
-                    raise LDAPSessionTerminatedByServer(self.connection.last_error)
+                    if log_enabled(ERROR):
+                        log(ERROR, '<%s> for <%s>', self.connection.last_error, self.connection)
+                    raise LDAPSessionTerminatedByServerError(self.connection.last_error)
 
                 # if referral in response opens a new connection to resolve referrals if requested
                 if responses[-2]['result'] == RESULT_REFERRAL:
@@ -298,10 +336,14 @@ class BaseStrategy(object):
                     break
 
             if timeout <= 0:
+                if log_enabled(ERROR):
+                    log(ERROR, 'socket timeout, no response from server for <%s>', self.connection)
                 raise LDAPResponseTimeoutError('no response from server')
 
             if self.connection.raise_exceptions and result and result['result'] not in DO_NOT_RAISE_EXCEPTIONS:
-                raise LDAPOperationResult(result=result['result'], description=result['description'], dn=result['dn'], message=result['message'], response_type=result['type'], response=response)
+                if log_enabled(PROTOCOL):
+                    log(PROTOCOL, 'operation result <%s> for <%s>', result, self.connection)
+                raise LDAPOperationResult(result=result['result'], description=result['description'], dn=result['dn'], message=result['message'], response_type=result['type'])
 
             # checks if any response has a range tag
             # self._auto_range_searching is set as a flag to avoid recursive searches
@@ -321,12 +363,14 @@ class BaseStrategy(object):
 
             self._outstanding.pop(message_id)
         else:
+            if log_enabled(ERROR):
+                log(ERROR, 'message id not in outstanding queue for <%s>', self.connection)
             raise(LDAPResponseTimeoutError('message id not in outstanding queue'))
 
         return response, result
 
-    @classmethod
-    def compute_ldap_message_size(cls, data):
+    @staticmethod
+    def compute_ldap_message_size(data):
         """
         Compute LDAP Message size according to BER definite length rules
         Returns -1 if too few data to compute message length
@@ -358,7 +402,10 @@ class BaseStrategy(object):
         component = ldap_message['protocolOp'].getComponent()
         controls = ldap_message['controls']
         if message_type == 'bindResponse':
-            result = bind_response_to_dict(component)
+            if not bytes(component['matchedDN']).startswith(b'NTLM'):  # patch for microsoft ntlm authentication
+                result = bind_response_to_dict(component)
+            else:
+                result = sicily_bind_response_to_dict(component)
         elif message_type == 'searchResEntry':
             result = search_result_entry_response_to_dict(component, self.connection.server.schema, self.connection.server.custom_formatter, self.connection.check_names)
         elif message_type == 'searchResDone':
@@ -380,6 +427,8 @@ class BaseStrategy(object):
         elif message_type == 'intermediateResponse':
             result = intermediate_response_to_dict(component)
         else:
+            if log_enabled(ERROR):
+                log(ERROR, 'unknown response <%s> for <%s>', message_type, self.connection)
             raise LDAPUnknownResponseError('unknown response')
         result['type'] = message_type
         if controls:
@@ -389,8 +438,59 @@ class BaseStrategy(object):
                 result['controls'][decoded_control[0]] = decoded_control[1]
         return result
 
-    @classmethod
-    def decode_control(cls, control):
+    def decode_response_fast(self, ldap_message):
+        """
+        Convert received LDAPMessage from fast ber decoder to a dict
+        """
+        if ldap_message['protocolOp'] == 1:  # bindResponse
+            if not ldap_message['payload'][1][3].startswith(b'NTLM'):  # patch for microsoft ntlm authentication
+                result = bind_response_to_dict_fast(ldap_message['payload'])
+            else:
+                result = sicily_bind_response_to_dict_fast(ldap_message['payload'])
+            result['type'] = 'bindResponse'
+        elif ldap_message['protocolOp'] == 4:  # searchResEntry'
+            result = search_result_entry_response_to_dict_fast(ldap_message['payload'], self.connection.server.schema, self.connection.server.custom_formatter, self.connection.check_names)
+            result['type'] = 'searchResEntry'
+        elif ldap_message['protocolOp'] == 5:  # searchResDone
+            result = ldap_result_to_dict_fast(ldap_message['payload'])
+            result['type'] = 'searchResDone'
+        elif ldap_message['protocolOp'] == 19:  # searchResRef
+            result = search_result_reference_response_to_dict_fast(ldap_message['payload'])
+            result['type'] = 'searchResRef'
+        elif ldap_message['protocolOp'] == 7:  # modifyResponse
+            result = ldap_result_to_dict_fast(ldap_message['payload'])
+            result['type'] = 'modifyResponse'
+        elif ldap_message['protocolOp'] == 9:  # addResponse
+            result = ldap_result_to_dict_fast(ldap_message['payload'])
+            result['type'] = 'addResponse'
+        elif ldap_message['protocolOp'] == 11:  # delResponse
+            result = ldap_result_to_dict_fast(ldap_message['payload'])
+            result['type'] = 'delResponse'
+        elif ldap_message['protocolOp'] == 13:  # modDNResponse
+            result = ldap_result_to_dict_fast(ldap_message['payload'])
+            result['type'] = 'modDNResponse'
+        elif ldap_message['protocolOp'] == 15:  # compareResponse
+            result = ldap_result_to_dict_fast(ldap_message['payload'])
+            result['type'] = 'compareResponse'
+        elif ldap_message['protocolOp'] == 24:  # extendedResp
+            result = extended_response_to_dict_fast(ldap_message['payload'])
+            result['type'] = 'extendedResp'
+        elif ldap_message['protocolOp'] == 25:  # intermediateResponse
+            result = intermediate_response_to_dict_fast(ldap_message['payload'])
+            result['type'] = 'intermediateResponse'
+        else:
+            if log_enabled(ERROR):
+                log(ERROR, 'unknown response <%s> for <%s>', ldap_message['protocolOp'], self.connection)
+            raise LDAPUnknownResponseError('unknown response')
+        if ldap_message['controls']:
+            result['controls'] = dict()
+            for control in ldap_message['controls']:
+                decoded_control = self.decode_control_fast(control[3])
+                result['controls'][decoded_control[0]] = decoded_control[1]
+        return result
+
+    @staticmethod
+    def decode_control(control):
         """
         decode control, return a 2-element tuple where the first element is the control oid
         and the second element is a dictionary with description (from Oids), criticality and decoded control value
@@ -398,18 +498,51 @@ class BaseStrategy(object):
         control_type = str(control['controlType'])
         criticality = bool(control['criticality'])
         control_value = bytes(control['controlValue'])
+        unprocessed = None
         if control_type == '1.2.840.113556.1.4.319':  # simple paged search as per RFC2696
             control_resp, unprocessed = decoder.decode(control_value, asn1Spec=RealSearchControlValue())
             control_value = dict()
             control_value['size'] = int(control_resp['size'])
             control_value['cookie'] = bytes(control_resp['cookie'])
-            if unprocessed:
-                raise LDAPControlsError('unprocessed control response in substrate for simple paged search')
-
+        elif control_type == '1.2.840.113556.1.4.841':  # DirSync AD
+            control_resp, unprocessed = decoder.decode(control_value, asn1Spec=DirSyncControlResponseValue())
+            control_value = dict()
+            control_value['more_results'] = bool(control_resp['MoreResults']) # more_result if nonzero
+            control_value['cookie'] = bytes(control_resp['CookieServer'])
+        if unprocessed:
+                if log_enabled(ERROR):
+                    log(ERROR, 'unprocessed control response in substrate')
+                raise LDAPControlsError('unprocessed control response in substrate')
         return control_type, {'description': Oids.get(control_type, ''), 'criticality': criticality, 'value': control_value}
 
-    @classmethod
-    def decode_request(cls, ldap_message):
+    @staticmethod
+    def decode_control_fast(control):
+        """
+        decode control, return a 2-element tuple where the first element is the control oid
+        and the second element is a dictionary with description (from Oids), criticality and decoded control value
+        """
+        control_type = str(control[0][3].decode('utf-8'))
+        criticality = False
+        control_value = None
+        for r in control[1:]:
+            if r[2] == 4:  # controlValue
+                control_value = r[3]
+            else:
+                criticality = False if r[3] == 0 else True  # criticality (booleand default to False)
+        if control_type == '1.2.840.113556.1.4.319':  # simple paged search as per RFC2696
+            control_resp = decode_sequence(control_value, 0, len(control_value))
+            control_value = dict()
+            control_value['size'] = int(control_resp[0][3][0][3])
+            control_value['cookie'] = bytes(control_resp[0][3][1][3])
+        elif control_type == '1.2.840.113556.1.4.841':  # DirSync AD
+            control_resp = decode_sequence(control_value, 0, len(control_value))
+            control_value = dict()
+            control_value['more_results'] = True if control_resp[0][3][0][3] else False  # more_result if nonzero
+            control_value['cookie'] = control_resp[0][3][2][3]
+        return control_type, {'description': Oids.get(control_type, ''), 'criticality': criticality, 'value': control_value}
+
+    @staticmethod
+    def decode_request(ldap_message):
         message_type = ldap_message.getComponentByName('protocolOp').getName()
         component = ldap_message['protocolOp'].getComponent()
         if message_type == 'bindRequest':
@@ -433,6 +566,8 @@ class BaseStrategy(object):
         elif message_type == 'abandonRequest':
             result = abandon_request_to_dict(component)
         else:
+            if log_enabled(ERROR):
+                log(ERROR, 'unknown request <%s>', message_type)
             raise LDAPUnknownRequestError('unknown request')
         result['type'] = message_type
         return result
@@ -460,6 +595,8 @@ class BaseStrategy(object):
             response['raw_attributes'][attr_type] += current_response['raw_attributes'][attr_name]
             response['attributes'][attr_type] += current_response['attributes'][attr_name]
             if high_range != '*':
+                if log_enabled(PROTOCOL):
+                    log(PROTOCOL, 'performing next search on auto-range <%s> via <%s>', str(int(high_range) + 1), self.connection)
                 result = self.connection.search(search_base=response['dn'],
                                                 search_filter='(objectclass=*)',
                                                 search_scope=BASE,
@@ -486,6 +623,8 @@ class BaseStrategy(object):
                     self.do_next_range_search(request, resp, attr_name)
 
     def do_operation_on_referral(self, request, referrals):
+        if log_enabled(PROTOCOL):
+            log(PROTOCOL, 'following referral for <%s>', self.connection)
         valid_referral_list = self.valid_referral_list(referrals)
         if valid_referral_list:
             preferred_referral_list = [referral for referral in valid_referral_list if referral['ssl'] == self.connection.server.ssl]
@@ -545,6 +684,8 @@ class BaseStrategy(object):
                                            controls=request['controls'])
             elif request['type'] == 'extendedRequest':
                 # TODO
+                if log_enabled(ERROR):
+                    log(ERROR, 'follow referrals on extended operation is not implemented for <%s>', self.connection)
                 raise NotImplementedError()
             elif request['type'] == 'modifyRequest':
                 referral_connection.modify(selected_referral['base'] or request['entry'],
@@ -558,6 +699,8 @@ class BaseStrategy(object):
                                               controls=request['controls'])
             else:
                 self.connection.last_error = 'referral operation not permitted'
+                if log_enabled(ERROR):
+                    log(ERROR, '<%s> for <%s>', self.connection.last_error, self.connection)
                 raise LDAPReferralError(self.connection.last_error)
 
             response = referral_connection.response
@@ -569,6 +712,30 @@ class BaseStrategy(object):
 
         return response, result
 
+    def sending(self, ldap_message):
+        exc = None
+        if log_enabled(NETWORK):
+            log(NETWORK, 'sending 1 ldap message for <%s>', self.connection)
+        try:
+            encoded_message = encoder.encode(ldap_message)
+            self.connection.socket.sendall(encoded_message)
+            if log_enabled(EXTENDED):
+                log(EXTENDED, 'ldap message sent via <%s>:%s', self.connection, format_ldap_message(ldap_message, '>>'))
+            if log_enabled(NETWORK):
+                log(NETWORK, 'sent %d bytes via <%s>', len(encoded_message), self.connection)
+        except socket.error as e:
+            self.connection.last_error = 'socket sending error' + str(e)
+            exc = e
+            encoded_message = None
+
+        if exc:
+            if log_enabled(ERROR):
+                log(ERROR, '<%s> for <%s>', self.connection.last_error, self.connection)
+            raise communication_exception_factory(LDAPSocketSendError, exc)(self.connection.last_error)
+
+        if self.connection.usage:
+            self.connection._usage.update_transmitted_message(self.connection.request, len(encoded_message))
+
     def _start_listen(self):
         # overridden on strategy class
         raise NotImplementedError
