@@ -23,9 +23,9 @@
 import unittest
 import ssl
 
-from ldap3 import Server, Connection, ServerPool, Tls, SASL
+from ldap3 import Server, Connection, ServerPool, Tls, SASL, EXTERNAL
 from test import test_server, test_port, test_port_ssl, test_user, test_password, test_authentication, \
-    test_strategy, test_lazy_connection, test_get_info, test_server_mode, \
+    test_strategy, test_lazy_connection, test_get_info, test_server_mode, test_valid_names, \
     test_pooling_strategy, test_pooling_active, test_pooling_exhaust, test_ca_cert_file, test_user_cert_file, test_user_key_file
 
 
@@ -99,14 +99,14 @@ class Test(unittest.TestCase):
                   validate=ssl.CERT_REQUIRED,
                   version=ssl.PROTOCOL_TLSv1,
                   ca_certs_file=test_ca_cert_file,
-                  valid_names=['EDIR-TEST', 'WIN1.FOREST.LAB', 'sles11sp3-template.hyperv'])
+                  valid_names=test_valid_names)
         if isinstance(test_server, (list, tuple)):
             server = ServerPool(pool_strategy=test_pooling_strategy, active=test_pooling_active, exhaust=test_pooling_exhaust)
             for host in test_server:
                 server.add(Server(host=host, port=test_port, allowed_referral_hosts=('*', True), get_info=test_get_info, mode=test_server_mode))
         else:
             server = Server(host=test_server, port=test_port_ssl, use_ssl=True, tls=tls)
-        connection = Connection(server, auto_bind=False, version=3, client_strategy=test_strategy, user=test_user, password=test_password, authentication=test_authentication)
+        connection = Connection(server, auto_bind=False, client_strategy=test_strategy, user=test_user, password=test_password, authentication=test_authentication)
         connection.open()
         connection.bind()
         self.assertTrue(connection.bound)
@@ -116,14 +116,14 @@ class Test(unittest.TestCase):
         self.assertFalse(connection.bound)
 
     def test_sasl_with_external_certificate(self):
-        tls = Tls(local_private_key_file=test_user_key_file, local_certificate_file=test_user_cert_file, validate=ssl.CERT_REQUIRED, version=ssl.PROTOCOL_TLSv1, ca_certs_file=test_ca_cert_file, valid_names=['EDIR-TEST', '2.hyperv', 'labldap02.cloudapp.net', 'WIN1.FOREST.LAB'])
+        tls = Tls(local_private_key_file=test_user_key_file, local_certificate_file=test_user_cert_file, validate=ssl.CERT_REQUIRED, version=ssl.PROTOCOL_TLSv1, ca_certs_file=test_ca_cert_file, valid_names=test_valid_names)
         if isinstance(test_server, (list, tuple)):
             server = ServerPool(pool_strategy=test_pooling_strategy, active=test_pooling_active, exhaust=test_pooling_exhaust)
             for host in test_server:
-                server.add(Server(host=host, port=test_port, allowed_referral_hosts=('*', True), get_info=test_get_info, mode=test_server_mode))
+                server.add(Server(host=host, port=test_port_ssl, use_ssl=True, tls=tls, allowed_referral_hosts=('*', True), get_info=test_get_info, mode=test_server_mode))
         else:
             server = Server(host=test_server, port=test_port_ssl, use_ssl=True, tls=tls)
-        connection = Connection(server, auto_bind=False, version=3, client_strategy=test_strategy, authentication=SASL, sasl_mechanism='EXTERNAL')
+        connection = Connection(server, auto_bind=False, version=3, client_strategy=test_strategy, authentication=SASL, sasl_mechanism=EXTERNAL)
         connection.open()
         connection.bind()
         self.assertTrue(connection.bound)
